@@ -26,13 +26,35 @@ namespace GtkSharp.Generation {
 
 	public class ByRefGen : SimpleBase {
 		
-		public ByRefGen (string ctype, string type) : base (ctype, type, type + ".Empty") {}
+		public ByRefGen (string ctype, string type) : base (ctype, type) {}
 		
 		public override string MarshalType {
 			get {
 				return "ref " + QualifiedName;
 			}
 		}
+
+		public override string MarshalReturnType {
+			get {
+				return QualifiedName;
+			}
+		}
+
+		public override string ToNativeReturnType {
+			get {
+				return QualifiedName;
+			}
+		}
+
+		public override string CallByName (string var_name)
+		{
+			return "ref " + var_name;
+		}
+		
+		public override string ToNativeReturn(string var)
+		{
+			return var;
+		}
 	}
 }
 
