@@ -31,56 +31,21 @@ namespace GLib {
 
 	public class Idle {
 
-		[UnmanagedFunctionPointer (CallingConvention.Cdecl)]
-		delegate bool IdleHandlerInternal (IntPtr ptr);
-
-
 		internal class IdleProxy : SourceProxy {
 			public IdleProxy (IdleHandler real)
 			{
 				real_handler = real;
-				proxy_handler = new IdleHandlerInternal (Handler);
+				proxy_handler = new IdleHandler (Handler);
 			}
 
-			~IdleProxy ()
+			public bool Handler ()
 			{
-				Dispose (false);
-			}
+				IdleHandler idle_handler = (IdleHandler) real_handler;
 
-			public void Dispose ()
-			{
-				Dispose (true);
-				GC.SuppressFinalize (this);
-			}
-
-			protected virtual void Dispose (bool disposing)
-			{
-				// Both branches remove our delegate from the
-				// managed list of handlers, but only
-				// Source.Remove will remove it from the
-
-				if (disposing)
+				bool cont = idle_handler ();
+				if (!cont)
 					Remove ();
-				else
-					Source.Remove (ID);
-			}
-
-			static bool Handler (IntPtr data)
-			{
-				try {
-					SourceProxy obj;
-					lock(proxies)
-						obj = proxies [(int)data];
-					IdleHandler idle_handler = (IdleHandler)obj.real_handler;
-
-					bool cont = idle_handler ();
-					if (!cont)
-						obj.Remove ();
-					return cont;
-				} catch (Exception e) {
-					ExceptionManager.RaiseUnhandledException (e, false);
-				}
-				return false;
+				return cont;
 			}
 		}
 		
@@ -88,42 +53,38 @@ namespace GLib {
 		{
 		}
 		
-		[DllImport("libglib-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
-		static extern uint g_idle_add (IdleHandlerInternal d, IntPtr data);
+		[DllImport("libglib-2.0-0.dll")]
+		static extern uint g_idle_add (IdleHandler d, IntPtr data);
 
 		public static uint Add (IdleHandler hndlr)
 		{
 			IdleProxy p = new IdleProxy (hndlr);
-			p.ID = g_idle_add ((IdleHandlerInternal)p.proxy_handler, (IntPtr)p.proxyId);
+			uint code = g_idle_add ((IdleHandler) p.proxy_handler, IntPtr.Zero);
 			lock (Source.source_handlers)
-				Source.source_handlers [p.ID] = p;
+				Source.source_handlers [code] = p;
 
-			return p.ID;
+			return code;
 		}
 		
-		[DllImport("libglib-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
-		static extern bool g_source_remove (uint id);
+		[DllImport("libglib-2.0-0.dll")]
+		static extern bool g_source_remove_by_funcs_user_data (Delegate d, IntPtr data);
                                                                                 
 		public static bool Remove (IdleHandler hndlr)
 		{
 			bool result = false;
-			var keys = new System.Collections.Generic.List<uint> ();
+			ArrayList keys = new ArrayList ();
 
 			lock (Source.source_handlers) {
 				foreach (uint code in Source.source_handlers.Keys) {
 					IdleProxy p = Source.source_handlers [code] as IdleProxy;
 				
-					if (p != null && p.real_handler == (System.Delegate) hndlr) {
+					if (p != null && p.real_handler == hndlr) {
 						keys.Add (code);
-						result = g_source_remove (code);
-						p.proxy_handler = null;
-						p.real_handler = null;
-						lock(SourceProxy.proxies)
-							SourceProxy.proxies.Remove (p.proxyId);
+						result = g_source_remove_by_funcs_user_data (p.proxy_handler, IntPtr.Zero);
 					}
 				}
 
-				foreach (var key in keys)
+				foreach (object key in keys)
 					Source.source_handlers.Remove (key);
 			}
 
